import * as i0 from '@angular/core';
import { Optional, EventEmitter, Component, Input, Output, ViewChildren, ViewChild, NgModule } from '@angular/core';
import * as i6 from '@angular/common';
import { CommonModule } from '@angular/common';
import { __decorate, __param } from 'tslib';
import * as i1 from '@angular/material/form-field';
import { MatFormFieldModule } from '@angular/material/form-field';
import * as i2 from '@angular/material/button';
import { MatButtonModule } from '@angular/material/button';
import * as i3 from '@angular/material/icon';
import { MatIconModule } from '@angular/material/icon';
import * as i4 from '@angular/material/progress-bar';
import { MatProgressBarModule } from '@angular/material/progress-bar';
import * as i5 from '@angular/material/radio';
import { MatRadioModule } from '@angular/material/radio';
import * as i6$1 from '@angular/material/checkbox';
import { MatCheckboxModule } from '@angular/material/checkbox';
import * as i7 from '@angular/material/divider';
import { MatDividerModule } from '@angular/material/divider';
import * as i10 from '@angular/material/input';
import { MatInputModule } from '@angular/material/input';
import * as i11 from '@angular/forms';
import { FormsModule } from '@angular/forms';
import * as i4$1 from '@angular/material/chips';
import { MatChipsModule } from '@angular/material/chips';
import { MatProgressSpinnerModule } from '@angular/material/progress-spinner';
import { MatTooltipModule } from '@angular/material/tooltip';

const SELECT_NONE = 0;
const SELECT_CHECKBOX = 1;
const SELECT_RADIO = 2;
/**
 * this class sets the config for complete tree
 */
class TreeLevelConfig {
    constructor(
    // make service call for children once or always
    _loadOnce = true, _allNode = true, _select = SELECT_NONE, _deleteNode = false, _addChild = false, _search = true) {
        this._loadOnce = _loadOnce;
        this._allNode = _allNode;
        this._select = _select;
        this._deleteNode = _deleteNode;
        this._addChild = _addChild;
        this._search = _search;
        this.loadChildrenStack = [];
        this._progress = false;
        this._searchStr = '';
    }
    get treeview() {
        return this._treeview;
    }
    set treeview(treeview) {
        this._treeview = treeview;
    }
    get loadOnce() {
        return this._loadOnce;
    }
    set loadOnce(loadOnce) {
        this._loadOnce = loadOnce;
    }
    get allNode() {
        return this._allNode;
    }
    set allNode(allNode) {
        this._allNode = allNode;
    }
    get select() {
        return this._select;
    }
    set select(select) {
        this._select = select;
    }
    get search() {
        return this._search;
    }
    set search(search) {
        this._search = search;
    }
    get searchStr() {
        return this._searchStr;
    }
    set searchStr(searchStr) {
        this._searchStr = searchStr;
    }
    get progress() {
        return this._progress;
    }
    set progress(progress) {
        this._progress = progress;
    }
    get deleteNode() {
        return this._deleteNode;
    }
    set deleteNode(deleteNode) {
        this._deleteNode = deleteNode;
    }
    get addChild() {
        return this._addChild;
    }
    set addChild(addChild) {
        this._addChild = addChild;
    }
    loadChildren() {
        this.loadChildrenStack.push(1);
        // console.log("push" + this.loadChildrenStack.length);
    }
    childrenLoaded() {
        this.loadChildrenStack.pop();
        if (this.loadChildrenStack.length == 0) {
            this.progress = false;
        }
        // console.log("pop" + this.loadChildrenStack.length);
    }
    checkloadChildrenStackSize() {
        if (this.loadChildrenStack.length == 0) {
            this.progress = false;
        }
    }
}

/**
 * this class public set the config for dropdown
 */
class DropdownLevelConfig {
    constructor(_height = 'auto', _showDropdownDefault = false) {
        this._height = _height;
        this._showDropdownDefault = _showDropdownDefault;
    }
    get height() {
        return this._height;
    }
    set height(height) {
        this._height = height;
    }
    get showDropdownDefault() {
        return this._showDropdownDefault;
    }
    set showDropdownDefault(showDropdownDefault) {
        this._showDropdownDefault = showDropdownDefault;
    }
}

class Config {
    constructor(_treeLevelConfig = new TreeLevelConfig(), _dropdownLevelConfig = new DropdownLevelConfig()) {
        this._treeLevelConfig = _treeLevelConfig;
        this._dropdownLevelConfig = _dropdownLevelConfig;
    }
    get treeLevelConfig() {
        return this._treeLevelConfig;
    }
    set treeLevelConfig(treeLevelConfig) {
        this._treeLevelConfig = treeLevelConfig;
    }
    get dropdownLevelConfig() {
        return this._dropdownLevelConfig;
    }
    set dropdownLevelConfig(dropdownLevelConfig) {
        this._dropdownLevelConfig = dropdownLevelConfig;
    }
}

const CHECKED = 1;
const UNCHECKED = 0;
const INDETERMINATE = -1;
/**
 * this class specifies the state(checked, collapsed, disabled) of the node
 */
class NodeState {
    constructor(_checked = UNCHECKED, _collapsed = true, _disabled = false, _hidden = false) {
        this._checked = _checked;
        this._collapsed = _collapsed;
        this._disabled = _disabled;
        this._hidden = _hidden;
    }
    get checked() {
        return this._checked;
    }
    set checked(checked) {
        this._checked = checked;
    }
    get collapsed() {
        return this._collapsed;
    }
    set collapsed(collapsed) {
        this._collapsed = collapsed;
    }
    get disabled() {
        return this._disabled;
    }
    set disabled(disabled) {
        this._disabled = disabled;
    }
    get hidden() {
        return this._hidden;
    }
    set hidden(hidden) {
        this._hidden = hidden;
    }
}

/**
 * this class properties if public set will override the tree level config
 */
let NodeLevelConfig = class NodeLevelConfig {
    constructor(_deleteNode, _addChild, _editNode, _renameNode) {
        this._deleteNode = _deleteNode;
        this._addChild = _addChild;
        this._editNode = _editNode;
        this._renameNode = _renameNode;
        if (this._addChild == undefined || this._addChild === undefined) {
            this._addChild = false;
        }
        if (this._deleteNode == undefined || this._deleteNode === undefined) {
            this._deleteNode = false;
        }
        if (this._editNode == undefined || this._editNode === undefined) {
            this._editNode = false;
        }
        if (this._renameNode == undefined || this._renameNode === undefined) {
            this._renameNode = false;
        }
    }
    get deleteNode() {
        return this._deleteNode;
    }
    set deleteNode(deleteNode) {
        this._deleteNode = deleteNode;
    }
    get addChild() {
        return this._addChild;
    }
    set addChild(addChild) {
        this._addChild = addChild;
    }
    get editNode() {
        return this._editNode;
    }
    set editNode(editNode) {
        this._editNode = editNode;
    }
    get renameNode() {
        return this._editNode;
    }
    set renameNode(renameNode) {
        this._renameNode = renameNode;
    }
};
NodeLevelConfig = __decorate([
    __param(0, Optional()),
    __param(1, Optional()),
    __param(2, Optional()),
    __param(3, Optional())
], NodeLevelConfig);

let Node = class Node {
    constructor(_name, _value, _children, _progress = false, _nodeState = new NodeState(), _nodeLevelConfig = new NodeLevelConfig(false, false, false, false)) {
        this._name = _name;
        this._value = _value;
        this._children = _children;
        this._progress = _progress;
        this._nodeState = _nodeState;
        this._nodeLevelConfig = _nodeLevelConfig;
        if (this._children == undefined || this._children === undefined) {
            this._children;
        }
    }
    /**
     * recursively sets prototype Node on the object and its children
     *
     * @param node
     */
    static nodify(obj) {
        const node = Object.setPrototypeOf(obj, Node.prototype);
        // nodeState
        if (node.nodeState == null || node.nodeState == undefined) {
            node.nodeState = new NodeState();
        }
        else {
            node.nodeState = Object.setPrototypeOf(node.nodeState, NodeState.prototype);
        }
        // nodeLevelConfig
        if (node.nodeLevelConfig == null || node.nodeLevelConfig == undefined) {
            node.nodeLevelConfig = new NodeLevelConfig(false, false, false, false);
        }
        else {
            node.nodeLevelConfig = Object.setPrototypeOf(node.nodeLevelConfig, NodeLevelConfig.prototype);
        }
        // children 
        if (node.children != null) {
            node.children.forEach(child => { this.nodify(child); child.parent = node; });
        }
        return node;
    }
    static toNodeArray(objArr) {
        let nodes = [];
        objArr.forEach(obj => {
            // console.log("obj.children");
            // console.log(obj.children);
            if (obj.children != null) {
                nodes.push(new Node(obj.name, obj.value, this.toNodeArray(obj.children), obj.progress, obj.nodeState, obj.nodeLevelConfig));
            }
            else {
                nodes.push(new Node(obj.name, obj.value, obj.children, obj.progress, obj.nodeState, obj.nodeLevelConfig));
            }
        });
        return nodes;
    }
    get name() {
        return this._name;
    }
    set name(name) {
        this._name = name;
    }
    get value() {
        return this._value;
    }
    set value(value) {
        this._value = value;
    }
    get children() {
        return this._children;
    }
    set children(children) {
        this._children = children;
    }
    loadChildren(children) {
        // children.forEach(child => Node.nodify(child));
        this._children = children;
        Node.nodify(this);
        this.setConfigRecursively(this.config);
        this._progress = false;
        this._nodeState.collapsed = false;
        this.verifyStateRecursive();
        //console.log(typeof this.config.treeLevelConfig.searchStr);
        if (this.config != null && this.config.treeLevelConfig != null && this.config.treeLevelConfig.searchStr != null && this.config.treeLevelConfig.searchStr != '') {
            this._children.forEach(child => child.filter(this.config.treeLevelConfig.searchStr, this.loadChildrenEvent));
        }
        if (this.config != null && this.config.treeLevelConfig != null)
            this.config.treeLevelConfig.childrenLoaded();
    }
    addChild(child) {
        if (this._children === null || this._children === undefined) {
            this._children = [];
        }
        this._children.push(Node.nodify(child));
        Node.nodify(this);
    }
    get progress() {
        return this._progress;
    }
    set progress(progress) {
        this._progress = progress;
    }
    get nodeState() {
        return this._nodeState;
    }
    set nodeState(nodeState) {
        this._nodeState = nodeState;
    }
    get nodeLevelConfig() {
        return this._nodeLevelConfig;
    }
    set nodeLevelConfig(nodeLevelConfig) {
        this._nodeLevelConfig = nodeLevelConfig;
    }
    get parent() {
        return this._parent;
    }
    set parent(node) {
        this._parent = node;
    }
    setConfigRecursively(config) {
        this.config = config;
        if (this.children != null) {
            this.children.forEach(child => child.setConfigRecursively(config));
        }
    }
    verifyStateRecursive() {
        if (this.children == null) {
            return;
        }
        if (this._nodeState.checked === CHECKED) {
            this.changeChildrenRecursive();
            return;
        }
        this.children.filter(n => n.children != null).forEach(n => n.verifyStateRecursive());
        this.checkImmediateChildren();
    }
    changeChildrenRecursive() {
        if (this.children == null) {
            return;
        }
        this.children.forEach(n => {
            n.nodeState.checked = this.nodeState.checked;
            n.changeChildrenRecursive();
        });
    }
    checkImmediateChildren() {
        // console.log(this.config.treeLevelConfig.customSearch);
        if (this.children && this.children.length > 0) {
            this.children.forEach(c => c.checkImmediateChildren());
            let checkedChildren = this.children.filter(n => n.nodeState.checked === CHECKED).length;
            let indeterminateChildren = this.children.filter(n => n.nodeState.checked === INDETERMINATE).length;
            if (indeterminateChildren > 0) {
                // if indeterminate child the indeterminate
                this.nodeState.checked = INDETERMINATE;
            }
            else {
                // if no indeterminate child
                if (checkedChildren === this.children.length) {
                    // if all checked then checked
                    if (window.localStorage.getItem("check") == "true") {
                    }
                    else {
                        this.nodeState.checked = CHECKED;
                    }
                    //this.nodeState.checked = CHECKED;
                }
                else if (checkedChildren === 0) {
                    // if all unchecked then unchecked
                    this.nodeState.checked = UNCHECKED;
                }
                else {
                    // if not all checked then indeterminate
                    this.nodeState.checked = INDETERMINATE;
                }
            }
        }
    }
    getCheckedValues() {
        if (this.nodeState.checked === CHECKED) {
            return [this];
        }
        if (this.children) {
            let values = [];
            this.children.forEach(n => {
                // console.log("getCheckedValues");
                n.getCheckedValues().forEach(v => values.push(v));
            });
            return values;
        }
        return [];
    }
    expandAndShowParentRecursively() {
        if (this.parent != null) {
            this.parent.nodeState.hidden = false;
            this.parent.nodeState.collapsed = false;
            this.parent.expandAndShowParentRecursively();
        }
        this.config.treeLevelConfig.checkloadChildrenStackSize();
    }
    filter(text, loadChildren) {
        this.loadChildrenEvent = loadChildren;
        if (this.children == null) {
            if (this.name.toLowerCase().indexOf(text.toLowerCase()) != -1) {
                this.nodeState.hidden = false;
                this.expandAndShowParentRecursively();
                return true;
            }
            else {
                this.nodeState.hidden = true;
                return false;
            }
        }
        else {
            if (this.children.length === 0) {
                this.progress = true;
                loadChildren.emit(this);
            }
            let matchFound = false;
            this.children.forEach(child => {
                let childMatchFound = child.filter(text, loadChildren);
                if (!matchFound) {
                    matchFound = childMatchFound;
                }
            });
            if (matchFound) {
                this.nodeState.hidden = false;
                this.nodeState.collapsed = false;
                this.expandAndShowParentRecursively();
                return true;
            }
            else {
                this.nodeState.collapsed = true;
                if (this.name.toLowerCase().indexOf(text.toLowerCase()) != -1) {
                    this.nodeState.hidden = false;
                    this.expandAndShowParentRecursively();
                    return true;
                }
                else {
                    this.nodeState.hidden = true;
                    return false;
                }
            }
        }
    }
    unHideChildren() {
        if (this.children) {
            this.children.forEach(child => {
                child.nodeState.hidden = false;
            });
        }
    }
    setCheckedRecursively(checked) {
        if (checked) {
            this.nodeState.checked = CHECKED;
        }
        else {
            this.nodeState.checked = UNCHECKED;
        }
        if (this.children) {
            this.children.forEach(child => {
                child.setCheckedRecursively(checked);
            });
        }
    }
    removeMe() {
        if (this.parent == null) {
            this.config.treeLevelConfig.treeview.nodes = this.config.treeLevelConfig.treeview.nodes.filter(node => node.value != this.value);
        }
        else {
            this.parent.children = this.parent.children.filter(child => child.value != this.value);
        }
    }
    search() {
        //console.log("Hello in search")
    }
};
Node = __decorate([
    __param(2, Optional())
], Node);

class SpTreeviewNodeTemplateContext {
    constructor() {
        this.node = new Node('', '', []);
        this.config = new Config();
        this.radioSelect = new EventEmitter;
        this.addChild = new EventEmitter;
        this.onCollapseExpand = (node) => {
            node.unHideChildren();
            if (node.nodeState.collapsed) {
                if ((this.config.treeLevelConfig.loadOnce && node.children.length === 0) || (!this.config.treeLevelConfig.loadOnce)) {
                    node.progress = true;
                    this.loadChildren.emit(node);
                    return;
                }
            }
            node.nodeState.collapsed = !node.nodeState.collapsed;
        };
        this.onDelete = (node) => {
            this.delete.emit(node);
        };
        this.onAddChild = (node) => {
            this.addChild.emit(node);
        };
        this.onLoadChildren = (node) => {
            this.loadChildren.emit(node);
        };
        this.onRadioChange = (event) => {
            this.radioSelect.emit([event.value]);
        };
        /**
         * called when the checkbox value is changed
         * sets checked value recursively
         */
        this.onCheckChange = (event) => {
            // set new check status for this node and its children
            // console.log(this.node.children);
            this.node.nodeState.checked = event.checked ? CHECKED : UNCHECKED;
            this.node.changeChildrenRecursive();
            // notify parent of the change
            this.selectParent.emit(this.node);
            this.checkboxSelect.emit();
        };
    }
}

class SpTreeviewNodeComponent {
    constructor() {
        this.SELECT_CHECKBOX = SELECT_CHECKBOX;
        this.SELECT_RADIO = SELECT_RADIO;
        this.SELECT_NONE = SELECT_NONE;
        this.CHECKED = CHECKED;
        this.UNCHECKED = UNCHECKED;
        this.INDETERMINATE = INDETERMINATE;
        this.config = new Config();
        this.radioSelect = new EventEmitter();
        this.checkboxSelect = new EventEmitter();
        this.delete = new EventEmitter();
        this.addChild = new EventEmitter();
        this.loadChildren = new EventEmitter();
        this.selectParent = new EventEmitter();
        this.dereference = new EventEmitter();
        this.context = new SpTreeviewNodeTemplateContext();
        this.renameNode = new EventEmitter();
        this.onDelete = (node) => {
            this.delete.emit(node);
        };
        this.onAddChild = (node) => {
            this.addChild.emit(node);
        };
        this.onLoadChildren = (node) => {
            this.loadChildren.emit(node);
        };
        this.onSelectParent = (node) => {
            this.selectParent.emit(node);
        };
    }
    ngOnInit() {
        // set input context prototype
        this.context = Object.setPrototypeOf(this.context, this.contextPrototype);
        // input
        this.context.node = this.node;
        this.context.config = this.config;
        // output
        this.context.addChild = this.addChild;
        this.context.delete = this.delete;
        this.context.loadChildren = this.loadChildren;
        this.context.checkboxSelect = this.checkboxSelect;
        this.context.radioSelect = this.radioSelect;
        this.context.selectParent = this.selectParent;
    }
    onRadioChange(nodes) {
        this.radioSelect.emit(nodes);
    }
    onCheckChange(nodes) {
        this.node.checkImmediateChildren();
        this.checkboxSelect.emit();
    }
    search(text) {
        return this.node.filter(text, this.loadChildren);
    }
    onDereference(node) {
        this.dereference.emit(node);
    }
    onRenameNode(node) {
        this.renameNode.emit(node);
    }
}
SpTreeviewNodeComponent.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.0.3", ngImport: i0, type: SpTreeviewNodeComponent, deps: [], target: i0.ɵɵFactoryTarget.Component });
SpTreeviewNodeComponent.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "12.0.0", version: "13.0.3", type: SpTreeviewNodeComponent, selector: "sp-treeview-node", inputs: { template: "template", contextPrototype: "contextPrototype", node: "node", config: "config" }, outputs: { radioSelect: "radioSelect", checkboxSelect: "checkboxSelect", delete: "delete", addChild: "addChild", loadChildren: "loadChildren", selectParent: "selectParent", dereference: "dereference", renameNode: "renameNode" }, ngImport: i0, template: "<div class=\"node-base\" [class.slideUp]=\"node.nodeState.hidden\">\n    <ng-template [ngTemplateOutlet]=\"template\" [ngTemplateOutletContext]=\"context\">\n    </ng-template>\n  \n    <!-- <div class=\"node-child\" *ngIf=\"!node.collapsed\"> -->\n    <div class=\"node-child\" [class.slideUp]=\"node.nodeState.collapsed\">\n      <sp-treeview-node *ngFor=\"let child of node.children\" [template]=\"template\" [contextPrototype]=\"contextPrototype\" [node]=\"child\"\n        [config]=\"config\" (radioSelect)=\"onRadioChange($event)\" (checkboxSelect)=\"onCheckChange($event);\" (delete)=\"onDelete($event)\"\n        (addChild)=\"onAddChild($event)\" (loadChildren)=\"onLoadChildren($event)\" (selectParent)=\"onSelectParent($event)\" (renameNode)=\"onRenameNode($event)\">\n      </sp-treeview-node>\n    </div>\n  </div>", styles: [".node-base{position:relative}.node-child{position:relative;padding-left:30px;overflow:hidden}.tree-node{height:40px;display:flex;align-items:center;justify-content:flex-start}.clickable{cursor:pointer}.hide{display:none}.slideUp{height:0px!important;padding-top:0;padding-bottom:0;overflow:hidden}*{transition:all .3s!important}.btn-small{width:24px!important;height:24px!important;line-height:24px!important}.icon-small{font-size:22px!important;line-height:22px!important}\n"], components: [{ type: SpTreeviewNodeComponent, selector: "sp-treeview-node", inputs: ["template", "contextPrototype", "node", "config"], outputs: ["radioSelect", "checkboxSelect", "delete", "addChild", "loadChildren", "selectParent", "dereference", "renameNode"] }], directives: [{ type: i6.NgTemplateOutlet, selector: "[ngTemplateOutlet]", inputs: ["ngTemplateOutletContext", "ngTemplateOutlet"] }, { type: i6.NgForOf, selector: "[ngFor][ngForOf]", inputs: ["ngForOf", "ngForTrackBy", "ngForTemplate"] }] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.0.3", ngImport: i0, type: SpTreeviewNodeComponent, decorators: [{
            type: Component,
            args: [{ selector: 'sp-treeview-node', template: "<div class=\"node-base\" [class.slideUp]=\"node.nodeState.hidden\">\n    <ng-template [ngTemplateOutlet]=\"template\" [ngTemplateOutletContext]=\"context\">\n    </ng-template>\n  \n    <!-- <div class=\"node-child\" *ngIf=\"!node.collapsed\"> -->\n    <div class=\"node-child\" [class.slideUp]=\"node.nodeState.collapsed\">\n      <sp-treeview-node *ngFor=\"let child of node.children\" [template]=\"template\" [contextPrototype]=\"contextPrototype\" [node]=\"child\"\n        [config]=\"config\" (radioSelect)=\"onRadioChange($event)\" (checkboxSelect)=\"onCheckChange($event);\" (delete)=\"onDelete($event)\"\n        (addChild)=\"onAddChild($event)\" (loadChildren)=\"onLoadChildren($event)\" (selectParent)=\"onSelectParent($event)\" (renameNode)=\"onRenameNode($event)\">\n      </sp-treeview-node>\n    </div>\n  </div>", styles: [".node-base{position:relative}.node-child{position:relative;padding-left:30px;overflow:hidden}.tree-node{height:40px;display:flex;align-items:center;justify-content:flex-start}.clickable{cursor:pointer}.hide{display:none}.slideUp{height:0px!important;padding-top:0;padding-bottom:0;overflow:hidden}*{transition:all .3s!important}.btn-small{width:24px!important;height:24px!important;line-height:24px!important}.icon-small{font-size:22px!important;line-height:22px!important}\n"] }]
        }], ctorParameters: function () { return []; }, propDecorators: { template: [{
                type: Input
            }], contextPrototype: [{
                type: Input
            }], node: [{
                type: Input
            }], config: [{
                type: Input
            }], radioSelect: [{
                type: Output
            }], checkboxSelect: [{
                type: Output
            }], delete: [{
                type: Output
            }], addChild: [{
                type: Output
            }], loadChildren: [{
                type: Output
            }], selectParent: [{
                type: Output
            }], dereference: [{
                type: Output
            }], renameNode: [{
                type: Output
            }] } });

class SpTreeviewComponent {
    constructor() {
        this.searchString = "";
        this.SELECT_CHECKBOX = SELECT_CHECKBOX;
        this.SELECT_RADIO = SELECT_RADIO;
        this.SELECT_NONE = SELECT_NONE;
        this.CHECKED = CHECKED;
        this.UNCHECKED = UNCHECKED;
        this.INDETERMINATE = INDETERMINATE;
        this.config = new Config();
        this.contextPrototype = SpTreeviewNodeTemplateContext.prototype;
        this.change = new EventEmitter();
        this.delete = new EventEmitter();
        this.addChild = new EventEmitter();
        this.loadChildren = new EventEmitter();
        this.selectParent = new EventEmitter();
        this.search = new EventEmitter();
        this.searchTo = new EventEmitter();
        this.dereference = new EventEmitter();
        this.editNode = new EventEmitter();
        this.renameNode = new EventEmitter();
    }
    ngOnInit() {
        if (this.customRootText == undefined || this.customRootText.length == 0) {
            this.all = new Node('All', 'ALL', []);
        }
        else {
            this.all = new Node(this.customRootText, 'ALL', []);
        }
        this.config.treeLevelConfig.treeview = this;
        this.nodes.forEach(n => {
            Node.nodify(n);
            n.setConfigRecursively(this.config);
        });
    }
    getSelectedValues() {
        let values = [];
        this.nodes.forEach(n => n.getCheckedValues().forEach(v => values.push(v)));
        return values;
    }
    onSearch(event) {
        if (this.customSearch) {
            this.searchTo.emit(event);
        }
    }
    applySearch(event) {
        this.config.treeLevelConfig.progress = true;
        if (this.customSearch) {
            this.search.emit(event);
        }
        else {
            this.trees.forEach(t => t.search(event));
        }
        setTimeout(() => { this.config.treeLevelConfig.progress = false; }, 1000);
    }
    onChange(nodes) {
        if (this.config.treeLevelConfig.select === SELECT_CHECKBOX) {
            let values = [];
            this.trees.forEach(t => {
                t.node.getCheckedValues().forEach((v) => values.push(v));
            });
            let isAllChecked = true;
            this.nodes.forEach(node => {
                if (node.nodeState.checked != CHECKED) {
                    isAllChecked = false;
                }
            });
            if (isAllChecked) {
                this.all.nodeState.checked = CHECKED;
                this.change.emit([this.all]);
            }
            else {
                this.all.nodeState.checked = UNCHECKED;
                this.change.emit(values);
            }
        }
        else if (this.config.treeLevelConfig.select === SELECT_RADIO) {
            this.all.nodeState.checked = UNCHECKED;
            this.change.emit(nodes);
        }
    }
    onDelete(node) {
        this.delete.emit(node);
    }
    onEditNode(node) {
        this.editNode.emit(node);
    }
    onAddChild(node) {
        this.addChild.emit(node);
    }
    onLoadChildren(node) {
        this.loadChildren.emit(node);
        this.config.treeLevelConfig.loadChildren();
    }
    onSelectParent(node) {
        this.selectParent.emit(node);
    }
    onAllRadioChange(event) {
        alert("onAllRadioChange");
        this.change.emit([this.all]);
        this.onSelectParent(this.all);
    }
    onAllCheckChange(event) {
        this.onSelectParent(this.all);
        this.nodes.forEach(node => {
            node.setCheckedRecursively(event.checked);
        });
        if (event.checked) {
            this.change.emit([this.all]);
        }
        else {
            this.change.emit([]);
        }
    }
    onAddRoot() {
        this.addChild.emit(this.all);
    }
    ngOnDestroy() {
    }
    onDereference(node) {
        this.dereference.emit(node);
    }
    onRenameNode(node) {
        this.renameNode.emit(node);
    }
}
SpTreeviewComponent.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.0.3", ngImport: i0, type: SpTreeviewComponent, deps: [], target: i0.ɵɵFactoryTarget.Component });
SpTreeviewComponent.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "12.0.0", version: "13.0.3", type: SpTreeviewComponent, selector: "sp-treeview", inputs: { nodes: "nodes", customSearch: "customSearch", customRootText: "customRootText", config: "config", template: "template", contextPrototype: "contextPrototype" }, outputs: { change: "change", delete: "delete", addChild: "addChild", loadChildren: "loadChildren", selectParent: "selectParent", search: "search", searchTo: "searchTo", dereference: "dereference", editNode: "editNode", renameNode: "renameNode" }, viewQueries: [{ propertyName: "trees", predicate: ["tree"], descendants: true }], ngImport: i0, template: "<div class=\"treeview-base\">\n    <div class=\"treeview-header\">\n      <mat-form-field *ngIf=\"config.treeLevelConfig.search\">\n        <!-- <input #search matInput placeholder=\"Type at least 3 characters\" (keyup)=\"onSearch($event)\" (change)=\"$event.stopPropagation()\"> -->\n        <input #search [(ngModel)]=\"searchString\" matInput (keyup)=\"onSearch(searchString)\" placeholder=\"Type at least 2 characters\" (change)=\"$event.stopPropagation()\">\n        <button (click)=\"applySearch(searchString)\" type=\"button\" mat-button mat-icon-button matPrefix color=\"primary\">\n          <mat-icon>search</mat-icon>\n        </button>\n        <!-- <button type=\"button\" mat-button mat-icon-button matSuffix color=\"warn\" (click)=\"$event.stopPropagation();search.value=null;applySearch('')\">\n          <mat-icon>close</mat-icon>\n        </button> -->\n      </mat-form-field>\n      <mat-progress-bar *ngIf=\"config.treeLevelConfig.progress\" mode=\"indeterminate\"></mat-progress-bar>\n    </div>\n    <div class=\"treeview-content\" [style.height]=\"config.dropdownLevelConfig.height\">\n      <mat-radio-group>\n        <div class=\"treeview-all\" *ngIf=\"config.treeLevelConfig.allNode\">\n          <span>\n            <!-- radio button -->\n            <mat-radio-button *ngIf=\"config.treeLevelConfig.select===SELECT_RADIO\" [value]=\"all\" [checked]=\"all.nodeState.checked===CHECKED\"\n              (change)=\"onAllRadioChange($event)\">\n              &nbsp;{{all.name}}&nbsp;&nbsp;\n            </mat-radio-button>\n            <!-- checkbox -->\n            <mat-checkbox *ngIf=\"config.treeLevelConfig.select===SELECT_CHECKBOX\" [value]=\"all.value\" [checked]=\"all.nodeState.checked===CHECKED\"\n              (change)=\"onAllCheckChange($event)\">\n              &nbsp;{{all.name}}&nbsp;&nbsp;\n            </mat-checkbox>\n            <!-- node name -->\n            <label *ngIf=\"config.treeLevelConfig.select===SELECT_NONE\">\n              &nbsp;{{all.name}}&nbsp;&nbsp;\n            </label>\n          <!-- </span>\n          <span> -->\n            <button mat-icon-button class=\"btn-small\" *ngIf=\"config.treeLevelConfig.addChild\" (click)=\"onAddRoot()\" title=\"Add\">\n              <mat-icon class=\"icon-small\" color=\"primary\">add</mat-icon>\n            </button>\n          </span>\n          <mat-divider></mat-divider>\n        </div>\n        <sp-treeview-node #tree *ngFor=\"let node of nodes;let i = index\" [template]=\"template?template:nodeTemplate\" [contextPrototype]=\"contextPrototype\"\n          [node]=\"node\" [config]=\"config\" (radioSelect)=\"onChange($event)\" (checkboxSelect)=\"onChange($event)\" (delete)=\"onDelete($event)\"\n          (addChild)=\"onAddChild($event)\" (editNode)=\"onEditNode($event)\" (selectParent)=\"onSelectParent($event)\" (loadChildren)=\"onLoadChildren($event)\" (renameNode)=\"onRenameNode($event)\"></sp-treeview-node>\n      </mat-radio-group>\n    </div>\n  </div>\n  \n  <link href=\"https://fonts.googleapis.com/icon?family=Material+Icons\" rel=\"stylesheet\">\n  <ng-template #nodeTemplate let-node=\"node\" let-config=\"config\" let-onCollapseExpand=\"onCollapseExpand\" let-onCheckChange=\"onCheckChange\"\n    let-onRadioChange=\"onRadioChange\" let-onDelete=\"onDelete\" let-onAddChild=\"onAddChild\" let-onLoadChildren=\"onLoadChildren\">\n    <div class=\"tree-node\">\n      <!-- expand/collapse button -->\n      <button type=\"button\" mat-button mat-icon-button *ngIf=\"node.children\" (click)=\"onCollapseExpand(node)\" color=\"primary\">\n        <mat-icon *ngIf=\"node.nodeState.collapsed\">keyboard_arrow_right</mat-icon>\n        <mat-icon *ngIf=\"!node.nodeState.collapsed\">keyboard_arrow_down</mat-icon>\n      </button>\n      <!-- if no children then empty space in place of button -->\n      <div style=\"width: 40px;height: 40px;flex-shrink: 0;\" *ngIf=\"node.children==null\"></div>\n      <!-- <button style=\"cursor: default;\" type=\"button\" mat-button mat-icon-button *ngIf=\"node.children==null\"></button> -->\n  \n      <!-- radio button -->\n      <mat-radio-button *ngIf=\"config.treeLevelConfig.select===SELECT_RADIO\" [checked]=\"node.nodeState.checked===CHECKED\" [value]=\"node\"\n        [disabled]=\"node.nodeState.disabled\" (change)=\"onRadioChange($event)\">\n        &nbsp;{{node.name}}&nbsp;&nbsp;\n      </mat-radio-button>\n      <!-- checkbox -->\n      <mat-checkbox *ngIf=\"config.treeLevelConfig.select===SELECT_CHECKBOX\" [checked]=\"node.nodeState.checked===CHECKED\" [value]=\"node.value\"\n        [indeterminate]=\"node.nodeState.checked===INDETERMINATE\" [disabled]=\"node.nodeState.disabled\" (change)=\"onCheckChange($event)\">\n        &nbsp;{{node.name}}&nbsp;&nbsp;\n      </mat-checkbox>\n  \n      <!-- node name -->\n      <label *ngIf=\"config.treeLevelConfig.select===SELECT_NONE\">\n        &nbsp;{{node.name}}&nbsp;&nbsp;\n      </label>\n  \n      <!-- add child -->\n      <button mat-icon-button class=\"btn-small\" *ngIf=\"((node.nodeLevelConfig.addChild!=null || node.nodeLevelConfig.addChild!=undefined) && node.nodeLevelConfig.addChild)\n       || ((node.nodeLevelConfig.addChild==null || node.nodeLevelConfig.addChild==undefined) && config.treeLevelConfig.addChild)\"\n        (click)=\"onAddChild(node)\" title=\"Add\">\n        <mat-icon class=\"icon-small\" color=\"primary\">add</mat-icon>\n      </button>\n  \n      <!-- edit node -->\n      <button mat-icon-button class=\"btn-small\" *ngIf=\"((node.nodeLevelConfig.editNode!=null || \n                                                          node.nodeLevelConfig.editNode!=undefined) && \n                                                          node.nodeLevelConfig.editNode)\"\n        (click)=\"onEditNode(node)\" title=\"Edit\">\n        <mat-icon class=\"icon-small\" color=\"primary\">edit</mat-icon>\n      </button>\n      <!-- rename node -->\n      <button mat-icon-button class=\"btn-small\"   *ngIf=\"((node.nodeLevelConfig.renameNode!=null || \n                                                            node.nodeLevelConfig.renameNode!=undefined) && \n                                                            node.nodeLevelConfig.renameNode) && node.value.isUserGroup=='true'\"\n        (click)=\"onRenameNode(node)\" title=\"Rename\">\n        <mat-icon class=\"icon-small custom-icon-green\">edit_note</mat-icon>\n      </button>\n  \n      <!-- delete node -->\n      <button mat-icon-button class=\"btn-small\" *ngIf=\"((node.nodeLevelConfig.deleteNode!=null || \n                                                          node.nodeLevelConfig.deleteNode!=undefined) && \n                                                          node.nodeLevelConfig.deleteNode) || \n                                                          ((node.nodeLevelConfig.deleteNode==null || \n                                                          node.nodeLevelConfig.deleteNode==undefined) && \n                                                          config.treeLevelConfig.deleteNode)\"\n        (click)=\"onDelete(node)\">\n        <!-- <mat-icon class=\"icon-small\" color=\"warn\">clear</mat-icon> -->\n        <mat-icon *ngIf=\"(node.value.isLeaf=='true') || (node.value.isLeaf=='false' && node.value.isUserGroup=='false')\" class=\"icon-small\" color=\"warn\" title=\"Unlink\">remove</mat-icon>  \n        <mat-icon *ngIf=\"node.value.isUserGroup=='true'\" class=\"icon-small\" color=\"warn\" title=\"remove\">clear</mat-icon> \n      </button>\n  \n      <!-- <button mat-icon-button class=\"btn-small red-color\" *ngIf=\"((node.nodeLevelConfig.deleteNode!=null || \n                                                                  node.nodeLevelConfig.deleteNode!=undefined) && \n                                                                  node.nodeLevelConfig.deleteNode) || \n                                                                  ((node.nodeLevelConfig.deleteNode==null \n                                                                  || node.nodeLevelConfig.deleteNode==undefined) && \n                                                                  config.treeLevelConfig.deleteNode) && \n                                                                  (node.value.isLeaf=='true') || \n                                                                  (node.value.isLeaf=='false' && \n                                                                  node.value.isUserGroup=='false')\" (click)=\"onDereference(node)\">\n        <i class=\"material-icons\">remove</i>\n      </button> -->\n  \n      <!-- progress bar -->\n      <mat-progress-bar *ngIf=\"node.progress\" mode=\"indeterminate\"></mat-progress-bar>\n    </div>\n  </ng-template>", styles: [".treeview-base{position:relative;min-width:-moz-fit-content;min-width:fit-content}mat-form-field{width:100%}.node-base{position:relative}.node-child{position:relative;padding-left:30px;overflow:hidden}.tree-node{height:40px;display:flex;align-items:center;justify-content:flex-start}.btn-small{width:24px!important;height:24px!important;line-height:24px!important}.icon-small{font-size:22px!important;line-height:22px!important}::ng-deep .mat-input-wrapper.mat-form-field-wrapper{padding-bottom:0}::ng-deep .mat-form-field-underline{bottom:0px!important}.treeview-content{overflow:auto}.treeview-all{padding:10px;display:flex;justify-content:space-between}.red-color{color:red}.custom-icon-green{color:#4caf50!important}\n"], components: [{ type: i1.MatFormField, selector: "mat-form-field", inputs: ["color", "appearance", "hideRequiredMarker", "hintLabel", "floatLabel"], exportAs: ["matFormField"] }, { type: i2.MatButton, selector: "button[mat-button], button[mat-raised-button], button[mat-icon-button],             button[mat-fab], button[mat-mini-fab], button[mat-stroked-button],             button[mat-flat-button]", inputs: ["disabled", "disableRipple", "color"], exportAs: ["matButton"] }, { type: i3.MatIcon, selector: "mat-icon", inputs: ["color", "inline", "svgIcon", "fontSet", "fontIcon"], exportAs: ["matIcon"] }, { type: i4.MatProgressBar, selector: "mat-progress-bar", inputs: ["color", "value", "bufferValue", "mode"], outputs: ["animationEnd"], exportAs: ["matProgressBar"] }, { type: i5.MatRadioButton, selector: "mat-radio-button", inputs: ["disableRipple", "tabIndex"], exportAs: ["matRadioButton"] }, { type: i6$1.MatCheckbox, selector: "mat-checkbox", inputs: ["disableRipple", "color", "tabIndex", "aria-label", "aria-labelledby", "aria-describedby", "id", "required", "labelPosition", "name", "value", "checked", "disabled", "indeterminate"], outputs: ["change", "indeterminateChange"], exportAs: ["matCheckbox"] }, { type: i7.MatDivider, selector: "mat-divider", inputs: ["vertical", "inset"] }, { type: SpTreeviewNodeComponent, selector: "sp-treeview-node", inputs: ["template", "contextPrototype", "node", "config"], outputs: ["radioSelect", "checkboxSelect", "delete", "addChild", "loadChildren", "selectParent", "dereference", "renameNode"] }], directives: [{ type: i6.NgIf, selector: "[ngIf]", inputs: ["ngIf", "ngIfThen", "ngIfElse"] }, { type: i10.MatInput, selector: "input[matInput], textarea[matInput], select[matNativeControl],      input[matNativeControl], textarea[matNativeControl]", inputs: ["disabled", "id", "placeholder", "name", "required", "type", "errorStateMatcher", "aria-describedby", "value", "readonly"], exportAs: ["matInput"] }, { type: i11.DefaultValueAccessor, selector: "input:not([type=checkbox])[formControlName],textarea[formControlName],input:not([type=checkbox])[formControl],textarea[formControl],input:not([type=checkbox])[ngModel],textarea[ngModel],[ngDefaultControl]" }, { type: i11.NgControlStatus, selector: "[formControlName],[ngModel],[formControl]" }, { type: i11.NgModel, selector: "[ngModel]:not([formControlName]):not([formControl])", inputs: ["name", "disabled", "ngModel", "ngModelOptions"], outputs: ["ngModelChange"], exportAs: ["ngModel"] }, { type: i1.MatPrefix, selector: "[matPrefix]" }, { type: i5.MatRadioGroup, selector: "mat-radio-group", exportAs: ["matRadioGroup"] }, { type: i6.NgForOf, selector: "[ngFor][ngForOf]", inputs: ["ngForOf", "ngForTrackBy", "ngForTemplate"] }] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.0.3", ngImport: i0, type: SpTreeviewComponent, decorators: [{
            type: Component,
            args: [{ selector: 'sp-treeview', template: "<div class=\"treeview-base\">\n    <div class=\"treeview-header\">\n      <mat-form-field *ngIf=\"config.treeLevelConfig.search\">\n        <!-- <input #search matInput placeholder=\"Type at least 3 characters\" (keyup)=\"onSearch($event)\" (change)=\"$event.stopPropagation()\"> -->\n        <input #search [(ngModel)]=\"searchString\" matInput (keyup)=\"onSearch(searchString)\" placeholder=\"Type at least 2 characters\" (change)=\"$event.stopPropagation()\">\n        <button (click)=\"applySearch(searchString)\" type=\"button\" mat-button mat-icon-button matPrefix color=\"primary\">\n          <mat-icon>search</mat-icon>\n        </button>\n        <!-- <button type=\"button\" mat-button mat-icon-button matSuffix color=\"warn\" (click)=\"$event.stopPropagation();search.value=null;applySearch('')\">\n          <mat-icon>close</mat-icon>\n        </button> -->\n      </mat-form-field>\n      <mat-progress-bar *ngIf=\"config.treeLevelConfig.progress\" mode=\"indeterminate\"></mat-progress-bar>\n    </div>\n    <div class=\"treeview-content\" [style.height]=\"config.dropdownLevelConfig.height\">\n      <mat-radio-group>\n        <div class=\"treeview-all\" *ngIf=\"config.treeLevelConfig.allNode\">\n          <span>\n            <!-- radio button -->\n            <mat-radio-button *ngIf=\"config.treeLevelConfig.select===SELECT_RADIO\" [value]=\"all\" [checked]=\"all.nodeState.checked===CHECKED\"\n              (change)=\"onAllRadioChange($event)\">\n              &nbsp;{{all.name}}&nbsp;&nbsp;\n            </mat-radio-button>\n            <!-- checkbox -->\n            <mat-checkbox *ngIf=\"config.treeLevelConfig.select===SELECT_CHECKBOX\" [value]=\"all.value\" [checked]=\"all.nodeState.checked===CHECKED\"\n              (change)=\"onAllCheckChange($event)\">\n              &nbsp;{{all.name}}&nbsp;&nbsp;\n            </mat-checkbox>\n            <!-- node name -->\n            <label *ngIf=\"config.treeLevelConfig.select===SELECT_NONE\">\n              &nbsp;{{all.name}}&nbsp;&nbsp;\n            </label>\n          <!-- </span>\n          <span> -->\n            <button mat-icon-button class=\"btn-small\" *ngIf=\"config.treeLevelConfig.addChild\" (click)=\"onAddRoot()\" title=\"Add\">\n              <mat-icon class=\"icon-small\" color=\"primary\">add</mat-icon>\n            </button>\n          </span>\n          <mat-divider></mat-divider>\n        </div>\n        <sp-treeview-node #tree *ngFor=\"let node of nodes;let i = index\" [template]=\"template?template:nodeTemplate\" [contextPrototype]=\"contextPrototype\"\n          [node]=\"node\" [config]=\"config\" (radioSelect)=\"onChange($event)\" (checkboxSelect)=\"onChange($event)\" (delete)=\"onDelete($event)\"\n          (addChild)=\"onAddChild($event)\" (editNode)=\"onEditNode($event)\" (selectParent)=\"onSelectParent($event)\" (loadChildren)=\"onLoadChildren($event)\" (renameNode)=\"onRenameNode($event)\"></sp-treeview-node>\n      </mat-radio-group>\n    </div>\n  </div>\n  \n  <link href=\"https://fonts.googleapis.com/icon?family=Material+Icons\" rel=\"stylesheet\">\n  <ng-template #nodeTemplate let-node=\"node\" let-config=\"config\" let-onCollapseExpand=\"onCollapseExpand\" let-onCheckChange=\"onCheckChange\"\n    let-onRadioChange=\"onRadioChange\" let-onDelete=\"onDelete\" let-onAddChild=\"onAddChild\" let-onLoadChildren=\"onLoadChildren\">\n    <div class=\"tree-node\">\n      <!-- expand/collapse button -->\n      <button type=\"button\" mat-button mat-icon-button *ngIf=\"node.children\" (click)=\"onCollapseExpand(node)\" color=\"primary\">\n        <mat-icon *ngIf=\"node.nodeState.collapsed\">keyboard_arrow_right</mat-icon>\n        <mat-icon *ngIf=\"!node.nodeState.collapsed\">keyboard_arrow_down</mat-icon>\n      </button>\n      <!-- if no children then empty space in place of button -->\n      <div style=\"width: 40px;height: 40px;flex-shrink: 0;\" *ngIf=\"node.children==null\"></div>\n      <!-- <button style=\"cursor: default;\" type=\"button\" mat-button mat-icon-button *ngIf=\"node.children==null\"></button> -->\n  \n      <!-- radio button -->\n      <mat-radio-button *ngIf=\"config.treeLevelConfig.select===SELECT_RADIO\" [checked]=\"node.nodeState.checked===CHECKED\" [value]=\"node\"\n        [disabled]=\"node.nodeState.disabled\" (change)=\"onRadioChange($event)\">\n        &nbsp;{{node.name}}&nbsp;&nbsp;\n      </mat-radio-button>\n      <!-- checkbox -->\n      <mat-checkbox *ngIf=\"config.treeLevelConfig.select===SELECT_CHECKBOX\" [checked]=\"node.nodeState.checked===CHECKED\" [value]=\"node.value\"\n        [indeterminate]=\"node.nodeState.checked===INDETERMINATE\" [disabled]=\"node.nodeState.disabled\" (change)=\"onCheckChange($event)\">\n        &nbsp;{{node.name}}&nbsp;&nbsp;\n      </mat-checkbox>\n  \n      <!-- node name -->\n      <label *ngIf=\"config.treeLevelConfig.select===SELECT_NONE\">\n        &nbsp;{{node.name}}&nbsp;&nbsp;\n      </label>\n  \n      <!-- add child -->\n      <button mat-icon-button class=\"btn-small\" *ngIf=\"((node.nodeLevelConfig.addChild!=null || node.nodeLevelConfig.addChild!=undefined) && node.nodeLevelConfig.addChild)\n       || ((node.nodeLevelConfig.addChild==null || node.nodeLevelConfig.addChild==undefined) && config.treeLevelConfig.addChild)\"\n        (click)=\"onAddChild(node)\" title=\"Add\">\n        <mat-icon class=\"icon-small\" color=\"primary\">add</mat-icon>\n      </button>\n  \n      <!-- edit node -->\n      <button mat-icon-button class=\"btn-small\" *ngIf=\"((node.nodeLevelConfig.editNode!=null || \n                                                          node.nodeLevelConfig.editNode!=undefined) && \n                                                          node.nodeLevelConfig.editNode)\"\n        (click)=\"onEditNode(node)\" title=\"Edit\">\n        <mat-icon class=\"icon-small\" color=\"primary\">edit</mat-icon>\n      </button>\n      <!-- rename node -->\n      <button mat-icon-button class=\"btn-small\"   *ngIf=\"((node.nodeLevelConfig.renameNode!=null || \n                                                            node.nodeLevelConfig.renameNode!=undefined) && \n                                                            node.nodeLevelConfig.renameNode) && node.value.isUserGroup=='true'\"\n        (click)=\"onRenameNode(node)\" title=\"Rename\">\n        <mat-icon class=\"icon-small custom-icon-green\">edit_note</mat-icon>\n      </button>\n  \n      <!-- delete node -->\n      <button mat-icon-button class=\"btn-small\" *ngIf=\"((node.nodeLevelConfig.deleteNode!=null || \n                                                          node.nodeLevelConfig.deleteNode!=undefined) && \n                                                          node.nodeLevelConfig.deleteNode) || \n                                                          ((node.nodeLevelConfig.deleteNode==null || \n                                                          node.nodeLevelConfig.deleteNode==undefined) && \n                                                          config.treeLevelConfig.deleteNode)\"\n        (click)=\"onDelete(node)\">\n        <!-- <mat-icon class=\"icon-small\" color=\"warn\">clear</mat-icon> -->\n        <mat-icon *ngIf=\"(node.value.isLeaf=='true') || (node.value.isLeaf=='false' && node.value.isUserGroup=='false')\" class=\"icon-small\" color=\"warn\" title=\"Unlink\">remove</mat-icon>  \n        <mat-icon *ngIf=\"node.value.isUserGroup=='true'\" class=\"icon-small\" color=\"warn\" title=\"remove\">clear</mat-icon> \n      </button>\n  \n      <!-- <button mat-icon-button class=\"btn-small red-color\" *ngIf=\"((node.nodeLevelConfig.deleteNode!=null || \n                                                                  node.nodeLevelConfig.deleteNode!=undefined) && \n                                                                  node.nodeLevelConfig.deleteNode) || \n                                                                  ((node.nodeLevelConfig.deleteNode==null \n                                                                  || node.nodeLevelConfig.deleteNode==undefined) && \n                                                                  config.treeLevelConfig.deleteNode) && \n                                                                  (node.value.isLeaf=='true') || \n                                                                  (node.value.isLeaf=='false' && \n                                                                  node.value.isUserGroup=='false')\" (click)=\"onDereference(node)\">\n        <i class=\"material-icons\">remove</i>\n      </button> -->\n  \n      <!-- progress bar -->\n      <mat-progress-bar *ngIf=\"node.progress\" mode=\"indeterminate\"></mat-progress-bar>\n    </div>\n  </ng-template>", styles: [".treeview-base{position:relative;min-width:-moz-fit-content;min-width:fit-content}mat-form-field{width:100%}.node-base{position:relative}.node-child{position:relative;padding-left:30px;overflow:hidden}.tree-node{height:40px;display:flex;align-items:center;justify-content:flex-start}.btn-small{width:24px!important;height:24px!important;line-height:24px!important}.icon-small{font-size:22px!important;line-height:22px!important}::ng-deep .mat-input-wrapper.mat-form-field-wrapper{padding-bottom:0}::ng-deep .mat-form-field-underline{bottom:0px!important}.treeview-content{overflow:auto}.treeview-all{padding:10px;display:flex;justify-content:space-between}.red-color{color:red}.custom-icon-green{color:#4caf50!important}\n"] }]
        }], ctorParameters: function () { return []; }, propDecorators: { nodes: [{
                type: Input
            }], customSearch: [{
                type: Input
            }], customRootText: [{
                type: Input
            }], config: [{
                type: Input
            }], template: [{
                type: Input
            }], contextPrototype: [{
                type: Input
            }], change: [{
                type: Output
            }], delete: [{
                type: Output
            }], addChild: [{
                type: Output
            }], loadChildren: [{
                type: Output
            }], selectParent: [{
                type: Output
            }], search: [{
                type: Output
            }], searchTo: [{
                type: Output
            }], dereference: [{
                type: Output
            }], editNode: [{
                type: Output
            }], trees: [{
                type: ViewChildren,
                args: ['tree']
            }], renameNode: [{
                type: Output
            }] } });

class SpTreeviewDropdownComponent {
    constructor() {
        this.config = new Config();
        this.contextPrototype = SpTreeviewNodeTemplateContext.prototype;
        this.change = new EventEmitter();
        this.delete = new EventEmitter();
        this.addChild = new EventEmitter();
        this.loadChildren = new EventEmitter();
        this.search = new EventEmitter();
        this.searchTo = new EventEmitter();
        this.selectParent = new EventEmitter();
        this.visible = true;
        this.selectable = false;
        this.removable = true;
        this.selectedNodes = [];
        this.dropDown = false;
        this.renameNode = new EventEmitter();
    }
    ngOnInit() {
        this.nodes.forEach(n => {
            n = Node.nodify(n);
            n.verifyStateRecursive();
            n.getCheckedValues().forEach(v => this.selectedNodes.push(v));
        });
        this.chipsDiv = this.chipList && this.chipList._elementRef.nativeElement.children[0];
        this.dropDown = this.config.dropdownLevelConfig.showDropdownDefault;
    }
    scrollLeft(event) {
        event.stopPropagation();
        this.chipsDiv.scrollLeft -= 50;
    }
    scrollRight(event) {
        event.stopPropagation();
        this.chipsDiv.scrollLeft += 50;
    }
    remove(node) {
        var _a, _b, _c, _d, _e, _f;
        this.dropDown = !this.dropDown;
        if ((node === null || node === void 0 ? void 0 : node.value) == 'ALL') {
            ((_a = this.nodes) !== null && _a !== void 0 ? _a : []).forEach(n => n.setCheckedRecursively(false));
            if (!this.selectedNodes)
                this.selectedNodes = [];
        }
        if (node === null || node === void 0 ? void 0 : node.nodeState) {
            node.nodeState.checked = UNCHECKED;
            ((_b = node.children) !== null && _b !== void 0 ? _b : []).forEach(n => n.nodeState.checked = UNCHECKED);
        }
        ((_c = this.nodes) !== null && _c !== void 0 ? _c : []).forEach(n => n.checkImmediateChildren());
        if (!this.selectedNodes)
            this.selectedNodes = [];
        let index = this.selectedNodes.findIndex(n => n === node);
        if (index !== -1) {
            this.selectedNodes.splice(index, 1);
        }
        let values = [];
        ((_e = (_d = this.tree) === null || _d === void 0 ? void 0 : _d.trees) !== null && _e !== void 0 ? _e : []).forEach(t => {
            var _a, _b;
            ((_b = (_a = t.node) === null || _a === void 0 ? void 0 : _a.getCheckedValues()) !== null && _b !== void 0 ? _b : []).forEach((v) => values.push(v));
        });
        (_f = this.change) === null || _f === void 0 ? void 0 : _f.emit(values);
    }
    onChange(nodes) {
        this.selectedNodes = nodes;
        this.change.emit(nodes);
    }
    onDelete(value) {
        this.delete.emit(value);
    }
    onAddChild(node) {
        this.addChild.emit(node);
    }
    onLoadChildren(node) {
        this.loadChildren.emit(node);
    }
    test(event) {
        this.searchTo.emit(event);
    }
    testSearch(event) {
        this.search.emit(event);
    }
    onSelectParent(node) {
        this.selectParent.emit(node);
    }
    onRenameNode(node) {
        this.renameNode.emit(node);
    }
}
SpTreeviewDropdownComponent.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.0.3", ngImport: i0, type: SpTreeviewDropdownComponent, deps: [], target: i0.ɵɵFactoryTarget.Component });
SpTreeviewDropdownComponent.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "12.0.0", version: "13.0.3", type: SpTreeviewDropdownComponent, selector: "sp-treeview-dropdown", inputs: { placeholder: "placeholder", customSearch: "customSearch", customRootText: "customRootText", required: "required", nodes: "nodes", config: "config", template: "template", contextPrototype: "contextPrototype", selectedNodes: "selectedNodes" }, outputs: { change: "change", delete: "delete", addChild: "addChild", loadChildren: "loadChildren", search: "search", searchTo: "searchTo", selectParent: "selectParent", renameNode: "renameNode" }, viewQueries: [{ propertyName: "tree", first: true, predicate: SpTreeviewComponent, descendants: true }, { propertyName: "chipList", first: true, predicate: ["chipList"], descendants: true }], ngImport: i0, template: "<link href=\"https://fonts.googleapis.com/icon?family=Material+Icons\" rel=\"stylesheet\">\n<div class=\"base\">\n  <!-- <mat-form-field class=\"full-width pointer\" [class.shrink]=\"selectedNodes==null || selectedNodes.length==0\" [class.liftup]=\"dropDown\" -->\n  <mat-form-field class=\"full-width pointer shrink\" [class.liftup]=\"dropDown\" (click)=\"dropDown=!dropDown\">\n\n    <button type=\"button\" matPrefix mat-icon-button (click)=\"scrollLeft($event)\" color=\"primary\">\n      <mat-icon>keyboard_arrow_left</mat-icon>\n    </button>\n\n    <mat-chip-list #chipList [required]=\"required\">\n      <mat-chip *ngFor=\"let node of selectedNodes\" [selectable]=\"selectable\" [removable]=\"removable\" (removed)=\"remove(node)\" (click)=\"$event.stopPropagation()\">\n        {{ node.name.replace(' ',\"&nbsp;\") }}\n        <mat-icon matChipRemove *ngIf=\"removable\">cancel</mat-icon>\n      </mat-chip>\n      <input mat-input [placeholder]=\"placeholder\" [matChipInputFor]=\"chipList\" [readonly]=true/>\n    </mat-chip-list>\n\n    <button type=\"button\" matSuffix mat-icon-button (click)=\"scrollRight($event)\" color=\"primary\">\n      <mat-icon>keyboard_arrow_right</mat-icon>\n    </button>\n    <button type=\"button\" matSuffix mat-icon-button color=\"primary\">\n      <mat-icon *ngIf=\"!dropDown\" matSuffix>arrow_drop_down</mat-icon>\n      <mat-icon *ngIf=\"dropDown\" matSuffix>arrow_drop_up</mat-icon>\n    </button>\n  </mat-form-field>\n  <!-- <div class=\"dropDown\" *ngIf=\"dropDown\" [style.height]=\"config.height\"> -->\n  <!-- <div class=\"dropDown\" [class.slideUp]=\"!dropDown\" [class.top-shrink]=\"selectedNodes==null || selectedNodes.length==0\"> -->\n  <div class=\"dropDown top-shrink\" [class.slideUp]=\"!dropDown\">\n    <sp-treeview [nodes]=\"nodes\" [config]=\"config\" (change)=\"onChange($event)\" (delete)=\"onDelete($event)\" (addChild)=\"onAddChild($event)\"\n      (loadChildren)=\"onLoadChildren($event)\" (search)= \"testSearch($event)\" (searchTo)=\"test($event)\" (selectParent)=\"onSelectParent($event)\"\n       [customSearch]=\"customSearch\" [customRootText]=\"customRootText\" (renameNode)=\"onRenameNode($event)\"></sp-treeview>\n\n    <div class=\"done\">\n      <button type=\"button\" matSuffix mat-raised-button color=\"primary\" (click)=\"dropDown=false\">\n        Done\n      </button>\n    </div>\n  </div>\n</div>\n\n<div class=\"cvr-screen\" *ngIf=\"dropDown\" (click)=\"$event.stopPropagation();dropDown=!dropDown;\"></div>\n\n", styles: [".base{position:relative}.full-width{width:100%}.pointer{cursor:pointer}.dropDown{background:white;position:absolute;padding:10px;top:60px;width:calc(100% - 20px);overflow:auto;box-shadow:0 1px 2px gray;z-index:1010}.dropDown>div.done{display:flex;justify-content:flex-end}.top-shrink{top:46px!important}.cvr-screen{position:fixed;top:0px;left:0px;width:100vw;height:100vh;z-index:1000}.liftup{z-index:1010}.shrink ::ng-deep .mat-chip-list-wrapper{flex-wrap:nowrap!important;align-items:center!important;overflow:hidden;scroll-behavior:smooth;height:32px}.shrink ::ng-deep .mat-chip{line-height:10px;font-size:10px;padding:1px 5px}.slideUp{height:0px!important;padding-top:0;padding-bottom:0}*{transition:all .3s}.shrink ::ng-deep .mat-input-infix{padding-top:0;padding-bottom:0}\n"], components: [{ type: i1.MatFormField, selector: "mat-form-field", inputs: ["color", "appearance", "hideRequiredMarker", "hintLabel", "floatLabel"], exportAs: ["matFormField"] }, { type: i2.MatButton, selector: "button[mat-button], button[mat-raised-button], button[mat-icon-button],             button[mat-fab], button[mat-mini-fab], button[mat-stroked-button],             button[mat-flat-button]", inputs: ["disabled", "disableRipple", "color"], exportAs: ["matButton"] }, { type: i3.MatIcon, selector: "mat-icon", inputs: ["color", "inline", "svgIcon", "fontSet", "fontIcon"], exportAs: ["matIcon"] }, { type: i4$1.MatChipList, selector: "mat-chip-list", inputs: ["errorStateMatcher", "multiple", "compareWith", "value", "required", "placeholder", "disabled", "aria-orientation", "selectable", "tabIndex"], outputs: ["change", "valueChange"], exportAs: ["matChipList"] }, { type: SpTreeviewComponent, selector: "sp-treeview", inputs: ["nodes", "customSearch", "customRootText", "config", "template", "contextPrototype"], outputs: ["change", "delete", "addChild", "loadChildren", "selectParent", "search", "searchTo", "dereference", "editNode", "renameNode"] }], directives: [{ type: i1.MatPrefix, selector: "[matPrefix]" }, { type: i6.NgForOf, selector: "[ngFor][ngForOf]", inputs: ["ngForOf", "ngForTrackBy", "ngForTemplate"] }, { type: i4$1.MatChip, selector: "mat-basic-chip, [mat-basic-chip], mat-chip, [mat-chip]", inputs: ["color", "disableRipple", "tabIndex", "selected", "value", "selectable", "disabled", "removable"], outputs: ["selectionChange", "destroyed", "removed"], exportAs: ["matChip"] }, { type: i6.NgIf, selector: "[ngIf]", inputs: ["ngIf", "ngIfThen", "ngIfElse"] }, { type: i4$1.MatChipRemove, selector: "[matChipRemove]" }, { type: i4$1.MatChipInput, selector: "input[matChipInputFor]", inputs: ["matChipInputFor", "matChipInputAddOnBlur", "matChipInputSeparatorKeyCodes", "placeholder", "id", "disabled"], outputs: ["matChipInputTokenEnd"], exportAs: ["matChipInput", "matChipInputFor"] }, { type: i1.MatSuffix, selector: "[matSuffix]" }] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.0.3", ngImport: i0, type: SpTreeviewDropdownComponent, decorators: [{
            type: Component,
            args: [{ selector: 'sp-treeview-dropdown', template: "<link href=\"https://fonts.googleapis.com/icon?family=Material+Icons\" rel=\"stylesheet\">\n<div class=\"base\">\n  <!-- <mat-form-field class=\"full-width pointer\" [class.shrink]=\"selectedNodes==null || selectedNodes.length==0\" [class.liftup]=\"dropDown\" -->\n  <mat-form-field class=\"full-width pointer shrink\" [class.liftup]=\"dropDown\" (click)=\"dropDown=!dropDown\">\n\n    <button type=\"button\" matPrefix mat-icon-button (click)=\"scrollLeft($event)\" color=\"primary\">\n      <mat-icon>keyboard_arrow_left</mat-icon>\n    </button>\n\n    <mat-chip-list #chipList [required]=\"required\">\n      <mat-chip *ngFor=\"let node of selectedNodes\" [selectable]=\"selectable\" [removable]=\"removable\" (removed)=\"remove(node)\" (click)=\"$event.stopPropagation()\">\n        {{ node.name.replace(' ',\"&nbsp;\") }}\n        <mat-icon matChipRemove *ngIf=\"removable\">cancel</mat-icon>\n      </mat-chip>\n      <input mat-input [placeholder]=\"placeholder\" [matChipInputFor]=\"chipList\" [readonly]=true/>\n    </mat-chip-list>\n\n    <button type=\"button\" matSuffix mat-icon-button (click)=\"scrollRight($event)\" color=\"primary\">\n      <mat-icon>keyboard_arrow_right</mat-icon>\n    </button>\n    <button type=\"button\" matSuffix mat-icon-button color=\"primary\">\n      <mat-icon *ngIf=\"!dropDown\" matSuffix>arrow_drop_down</mat-icon>\n      <mat-icon *ngIf=\"dropDown\" matSuffix>arrow_drop_up</mat-icon>\n    </button>\n  </mat-form-field>\n  <!-- <div class=\"dropDown\" *ngIf=\"dropDown\" [style.height]=\"config.height\"> -->\n  <!-- <div class=\"dropDown\" [class.slideUp]=\"!dropDown\" [class.top-shrink]=\"selectedNodes==null || selectedNodes.length==0\"> -->\n  <div class=\"dropDown top-shrink\" [class.slideUp]=\"!dropDown\">\n    <sp-treeview [nodes]=\"nodes\" [config]=\"config\" (change)=\"onChange($event)\" (delete)=\"onDelete($event)\" (addChild)=\"onAddChild($event)\"\n      (loadChildren)=\"onLoadChildren($event)\" (search)= \"testSearch($event)\" (searchTo)=\"test($event)\" (selectParent)=\"onSelectParent($event)\"\n       [customSearch]=\"customSearch\" [customRootText]=\"customRootText\" (renameNode)=\"onRenameNode($event)\"></sp-treeview>\n\n    <div class=\"done\">\n      <button type=\"button\" matSuffix mat-raised-button color=\"primary\" (click)=\"dropDown=false\">\n        Done\n      </button>\n    </div>\n  </div>\n</div>\n\n<div class=\"cvr-screen\" *ngIf=\"dropDown\" (click)=\"$event.stopPropagation();dropDown=!dropDown;\"></div>\n\n", styles: [".base{position:relative}.full-width{width:100%}.pointer{cursor:pointer}.dropDown{background:white;position:absolute;padding:10px;top:60px;width:calc(100% - 20px);overflow:auto;box-shadow:0 1px 2px gray;z-index:1010}.dropDown>div.done{display:flex;justify-content:flex-end}.top-shrink{top:46px!important}.cvr-screen{position:fixed;top:0px;left:0px;width:100vw;height:100vh;z-index:1000}.liftup{z-index:1010}.shrink ::ng-deep .mat-chip-list-wrapper{flex-wrap:nowrap!important;align-items:center!important;overflow:hidden;scroll-behavior:smooth;height:32px}.shrink ::ng-deep .mat-chip{line-height:10px;font-size:10px;padding:1px 5px}.slideUp{height:0px!important;padding-top:0;padding-bottom:0}*{transition:all .3s}.shrink ::ng-deep .mat-input-infix{padding-top:0;padding-bottom:0}\n"] }]
        }], ctorParameters: function () { return []; }, propDecorators: { placeholder: [{
                type: Input
            }], customSearch: [{
                type: Input
            }], customRootText: [{
                type: Input
            }], required: [{
                type: Input
            }], nodes: [{
                type: Input
            }], config: [{
                type: Input
            }], template: [{
                type: Input
            }], contextPrototype: [{
                type: Input
            }], change: [{
                type: Output
            }], delete: [{
                type: Output
            }], addChild: [{
                type: Output
            }], loadChildren: [{
                type: Output
            }], search: [{
                type: Output
            }], searchTo: [{
                type: Output
            }], selectParent: [{
                type: Output
            }], tree: [{
                type: ViewChild,
                args: [SpTreeviewComponent]
            }], chipList: [{
                type: ViewChild,
                args: ['chipList']
            }], selectedNodes: [{
                type: Input
            }], renameNode: [{
                type: Output
            }] } });

class SpTreeviewOverlayComponent {
    constructor() {
        this.config = new Config();
        this.contextPrototype = null;
        this.change = new EventEmitter();
        this.delete = new EventEmitter();
        this.addChild = new EventEmitter();
        this.loadChildren = new EventEmitter();
        this.visible = true;
        this.selectable = false;
        this.removable = true;
        this.selectedNodes = [];
        this.dropDown = false;
        this.renameNode = new EventEmitter();
    }
    ngOnInit() {
        this.nodes.forEach(n => {
            n = Node.nodify(n);
            n.verifyStateRecursive();
            n.getCheckedValues().forEach(v => this.selectedNodes.push(v));
        });
        this.config.dropdownLevelConfig.height = 'calc(100vh - 125px)';
        this.chipsDiv = this.chipList && this.chipList._elementRef.nativeElement.children[0];
        this.dropDown = this.config.dropdownLevelConfig.showDropdownDefault;
    }
    scrollLeft(event) {
        event.stopPropagation();
        this.chipsDiv.scrollLeft -= 50;
    }
    scrollRight(event) {
        event.stopPropagation();
        this.chipsDiv.scrollLeft += 50;
    }
    remove(node) {
        var _a, _b, _c, _d, _e, _f;
        this.dropDown = !this.dropDown;
        if ((node === null || node === void 0 ? void 0 : node.value) == 'ALL') {
            ((_a = this.nodes) !== null && _a !== void 0 ? _a : []).forEach(n => n.setCheckedRecursively(false));
            if (!this.selectedNodes)
                this.selectedNodes = [];
        }
        if (node === null || node === void 0 ? void 0 : node.nodeState) {
            node.nodeState.checked = UNCHECKED;
            ((_b = node.children) !== null && _b !== void 0 ? _b : []).forEach(n => n.nodeState.checked = UNCHECKED);
        }
        ((_c = this.nodes) !== null && _c !== void 0 ? _c : []).forEach(n => n.checkImmediateChildren());
        if (!this.selectedNodes)
            this.selectedNodes = [];
        let index = this.selectedNodes.findIndex(n => n === node);
        if (index !== -1) {
            this.selectedNodes.splice(index, 1);
        }
        let values = [];
        ((_e = (_d = this.tree) === null || _d === void 0 ? void 0 : _d.trees) !== null && _e !== void 0 ? _e : []).forEach(t => {
            var _a, _b;
            ((_b = (_a = t.node) === null || _a === void 0 ? void 0 : _a.getCheckedValues()) !== null && _b !== void 0 ? _b : []).forEach((v) => values.push(v));
        });
        (_f = this.change) === null || _f === void 0 ? void 0 : _f.emit(values);
    }
    onChange(nodes) {
        this.selectedNodes = nodes;
        this.change.emit(nodes);
    }
    onDelete(value) {
        this.delete.emit(value);
    }
    onAddChild(node) {
        this.addChild.emit(node);
    }
    onLoadChildren(node) {
        this.loadChildren.emit(node);
    }
    onRenameNode(node) {
        this.renameNode.emit(node);
    }
}
SpTreeviewOverlayComponent.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.0.3", ngImport: i0, type: SpTreeviewOverlayComponent, deps: [], target: i0.ɵɵFactoryTarget.Component });
SpTreeviewOverlayComponent.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "12.0.0", version: "13.0.3", type: SpTreeviewOverlayComponent, selector: "sp-treeview-overlay", inputs: { placeholder: "placeholder", required: "required", nodes: "nodes", config: "config", template: "template", contextPrototype: "contextPrototype" }, outputs: { change: "change", delete: "delete", addChild: "addChild", loadChildren: "loadChildren", renameNode: "renameNode" }, viewQueries: [{ propertyName: "tree", first: true, predicate: SpTreeviewComponent, descendants: true }, { propertyName: "chipList", first: true, predicate: ["chipList"], descendants: true }], ngImport: i0, template: "<link href=\"https://fonts.googleapis.com/icon?family=Material+Icons\" rel=\"stylesheet\">\n<div class=\"base\">\n  <!-- <mat-form-field class=\"full-width pointer\" [class.shrink]=\"selectedNodes==null || selectedNodes.length==0\" [class.liftup]=\"dropDown\" -->\n  <mat-form-field class=\"full-width pointer shrink\" [class.liftup]=\"dropDown\" (click)=\"dropDown=!dropDown\">\n\n    <button type=\"button\" matPrefix mat-icon-button (click)=\"scrollLeft($event)\" color=\"primary\">\n      <mat-icon>keyboard_arrow_left</mat-icon>\n    </button>\n\n    <mat-chip-list #chipList [required]=\"required\">\n      <mat-chip *ngFor=\"let node of selectedNodes\" [selectable]=\"selectable\" [removable]=\"removable\" (removed)=\"remove(node)\" (click)=\"$event.stopPropagation()\">\n        {{ node.name.replace(' ',\"&nbsp;\") }}\n        <mat-icon matChipRemove *ngIf=\"removable\">cancel</mat-icon>\n      </mat-chip>\n      <input mat-input [placeholder]=\"placeholder\" [matChipInputFor]=\"chipList\" [readonly]=true/>\n    </mat-chip-list>\n\n    <button type=\"button\" matSuffix mat-icon-button (click)=\"scrollRight($event)\" color=\"primary\">\n      <mat-icon>keyboard_arrow_right</mat-icon>\n    </button>\n    <button type=\"button\" matSuffix mat-icon-button color=\"primary\">\n      <mat-icon *ngIf=\"!dropDown\" matSuffix>arrow_drop_down</mat-icon>\n      <mat-icon *ngIf=\"dropDown\" matSuffix>arrow_drop_up</mat-icon>\n    </button>\n  </mat-form-field>\n  <!-- <div class=\"dropDown\" *ngIf=\"dropDown\" [style.height]=\"config.height\"> -->\n  <div class=\"overlay\" [class.slideUp]=\"!dropDown\">\n    <div style=\"height: calc(100% - 40px);overflow: auto;\">\n      <sp-treeview [template]=\"template\" [contextPrototype]=\"contextPrototype\" [nodes]=\"nodes\" [config]=\"config\" (change)=\"onChange($event)\"\n        (delete)=\"onDelete($event)\" (addChild)=\"onAddChild($event)\" (loadChildren)=\"onLoadChildren($event)\" (renameNode)=\"onRenameNode($event)\"></sp-treeview>\n    </div>\n    <div class=\"done\">\n      <button type=\"button\" matSuffix mat-raised-button color=\"primary\" (click)=\"dropDown=false\">\n        Done\n      </button>\n    </div>\n  </div>\n</div>\n\n<div class=\"cvr-screen\" *ngIf=\"dropDown\" (click)=\"$event.stopPropagation();dropDown=!dropDown;\"></div>", styles: [".base{position:relative}.full-width{width:100%}.pointer{cursor:pointer}.dropDown{background:white;position:absolute;padding:10px;top:60px;width:calc(100% - 20px);overflow:auto;box-shadow:0 1px 2px gray;z-index:1010}.overlay{position:fixed;background:white;padding:10px;width:calc(100% - 40px);height:calc(100% - 40px);top:10px;left:10px;overflow:auto;box-shadow:0 1px 2px gray;z-index:1010}.overlay>div.done{display:flex;justify-content:flex-end}.cvr-screen{position:fixed;top:0px;left:0px;width:100vw;height:100vh;z-index:1000}.liftup{z-index:1010}.shrink ::ng-deep .mat-chip-list-wrapper{flex-wrap:nowrap!important;align-items:center!important;overflow:hidden;scroll-behavior:smooth;height:32px}.shrink ::ng-deep .mat-chip{line-height:10px;font-size:10px;padding:1px 5px}.slideUp{height:0px!important;padding-top:0;padding-bottom:0}*{transition:all .3s}.shrink ::ng-deep .mat-input-infix{padding-top:0;padding-bottom:0}\n"], components: [{ type: i1.MatFormField, selector: "mat-form-field", inputs: ["color", "appearance", "hideRequiredMarker", "hintLabel", "floatLabel"], exportAs: ["matFormField"] }, { type: i2.MatButton, selector: "button[mat-button], button[mat-raised-button], button[mat-icon-button],             button[mat-fab], button[mat-mini-fab], button[mat-stroked-button],             button[mat-flat-button]", inputs: ["disabled", "disableRipple", "color"], exportAs: ["matButton"] }, { type: i3.MatIcon, selector: "mat-icon", inputs: ["color", "inline", "svgIcon", "fontSet", "fontIcon"], exportAs: ["matIcon"] }, { type: i4$1.MatChipList, selector: "mat-chip-list", inputs: ["errorStateMatcher", "multiple", "compareWith", "value", "required", "placeholder", "disabled", "aria-orientation", "selectable", "tabIndex"], outputs: ["change", "valueChange"], exportAs: ["matChipList"] }, { type: SpTreeviewComponent, selector: "sp-treeview", inputs: ["nodes", "customSearch", "customRootText", "config", "template", "contextPrototype"], outputs: ["change", "delete", "addChild", "loadChildren", "selectParent", "search", "searchTo", "dereference", "editNode", "renameNode"] }], directives: [{ type: i1.MatPrefix, selector: "[matPrefix]" }, { type: i6.NgForOf, selector: "[ngFor][ngForOf]", inputs: ["ngForOf", "ngForTrackBy", "ngForTemplate"] }, { type: i4$1.MatChip, selector: "mat-basic-chip, [mat-basic-chip], mat-chip, [mat-chip]", inputs: ["color", "disableRipple", "tabIndex", "selected", "value", "selectable", "disabled", "removable"], outputs: ["selectionChange", "destroyed", "removed"], exportAs: ["matChip"] }, { type: i6.NgIf, selector: "[ngIf]", inputs: ["ngIf", "ngIfThen", "ngIfElse"] }, { type: i4$1.MatChipRemove, selector: "[matChipRemove]" }, { type: i4$1.MatChipInput, selector: "input[matChipInputFor]", inputs: ["matChipInputFor", "matChipInputAddOnBlur", "matChipInputSeparatorKeyCodes", "placeholder", "id", "disabled"], outputs: ["matChipInputTokenEnd"], exportAs: ["matChipInput", "matChipInputFor"] }, { type: i1.MatSuffix, selector: "[matSuffix]" }] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.0.3", ngImport: i0, type: SpTreeviewOverlayComponent, decorators: [{
            type: Component,
            args: [{ selector: 'sp-treeview-overlay', template: "<link href=\"https://fonts.googleapis.com/icon?family=Material+Icons\" rel=\"stylesheet\">\n<div class=\"base\">\n  <!-- <mat-form-field class=\"full-width pointer\" [class.shrink]=\"selectedNodes==null || selectedNodes.length==0\" [class.liftup]=\"dropDown\" -->\n  <mat-form-field class=\"full-width pointer shrink\" [class.liftup]=\"dropDown\" (click)=\"dropDown=!dropDown\">\n\n    <button type=\"button\" matPrefix mat-icon-button (click)=\"scrollLeft($event)\" color=\"primary\">\n      <mat-icon>keyboard_arrow_left</mat-icon>\n    </button>\n\n    <mat-chip-list #chipList [required]=\"required\">\n      <mat-chip *ngFor=\"let node of selectedNodes\" [selectable]=\"selectable\" [removable]=\"removable\" (removed)=\"remove(node)\" (click)=\"$event.stopPropagation()\">\n        {{ node.name.replace(' ',\"&nbsp;\") }}\n        <mat-icon matChipRemove *ngIf=\"removable\">cancel</mat-icon>\n      </mat-chip>\n      <input mat-input [placeholder]=\"placeholder\" [matChipInputFor]=\"chipList\" [readonly]=true/>\n    </mat-chip-list>\n\n    <button type=\"button\" matSuffix mat-icon-button (click)=\"scrollRight($event)\" color=\"primary\">\n      <mat-icon>keyboard_arrow_right</mat-icon>\n    </button>\n    <button type=\"button\" matSuffix mat-icon-button color=\"primary\">\n      <mat-icon *ngIf=\"!dropDown\" matSuffix>arrow_drop_down</mat-icon>\n      <mat-icon *ngIf=\"dropDown\" matSuffix>arrow_drop_up</mat-icon>\n    </button>\n  </mat-form-field>\n  <!-- <div class=\"dropDown\" *ngIf=\"dropDown\" [style.height]=\"config.height\"> -->\n  <div class=\"overlay\" [class.slideUp]=\"!dropDown\">\n    <div style=\"height: calc(100% - 40px);overflow: auto;\">\n      <sp-treeview [template]=\"template\" [contextPrototype]=\"contextPrototype\" [nodes]=\"nodes\" [config]=\"config\" (change)=\"onChange($event)\"\n        (delete)=\"onDelete($event)\" (addChild)=\"onAddChild($event)\" (loadChildren)=\"onLoadChildren($event)\" (renameNode)=\"onRenameNode($event)\"></sp-treeview>\n    </div>\n    <div class=\"done\">\n      <button type=\"button\" matSuffix mat-raised-button color=\"primary\" (click)=\"dropDown=false\">\n        Done\n      </button>\n    </div>\n  </div>\n</div>\n\n<div class=\"cvr-screen\" *ngIf=\"dropDown\" (click)=\"$event.stopPropagation();dropDown=!dropDown;\"></div>", styles: [".base{position:relative}.full-width{width:100%}.pointer{cursor:pointer}.dropDown{background:white;position:absolute;padding:10px;top:60px;width:calc(100% - 20px);overflow:auto;box-shadow:0 1px 2px gray;z-index:1010}.overlay{position:fixed;background:white;padding:10px;width:calc(100% - 40px);height:calc(100% - 40px);top:10px;left:10px;overflow:auto;box-shadow:0 1px 2px gray;z-index:1010}.overlay>div.done{display:flex;justify-content:flex-end}.cvr-screen{position:fixed;top:0px;left:0px;width:100vw;height:100vh;z-index:1000}.liftup{z-index:1010}.shrink ::ng-deep .mat-chip-list-wrapper{flex-wrap:nowrap!important;align-items:center!important;overflow:hidden;scroll-behavior:smooth;height:32px}.shrink ::ng-deep .mat-chip{line-height:10px;font-size:10px;padding:1px 5px}.slideUp{height:0px!important;padding-top:0;padding-bottom:0}*{transition:all .3s}.shrink ::ng-deep .mat-input-infix{padding-top:0;padding-bottom:0}\n"] }]
        }], ctorParameters: function () { return []; }, propDecorators: { placeholder: [{
                type: Input
            }], required: [{
                type: Input
            }], nodes: [{
                type: Input
            }], config: [{
                type: Input
            }], template: [{
                type: Input
            }], contextPrototype: [{
                type: Input
            }], change: [{
                type: Output
            }], delete: [{
                type: Output
            }], addChild: [{
                type: Output
            }], loadChildren: [{
                type: Output
            }], tree: [{
                type: ViewChild,
                args: [SpTreeviewComponent]
            }], chipList: [{
                type: ViewChild,
                args: ['chipList']
            }], renameNode: [{
                type: Output
            }] } });

const modules = [
    MatProgressBarModule,
    MatCheckboxModule,
    MatRadioModule,
    MatButtonModule,
    MatIconModule,
    MatProgressBarModule,
    MatProgressSpinnerModule,
    MatDividerModule,
    MatFormFieldModule,
    MatChipsModule,
    MatInputModule,
    MatTooltipModule
    // BrowserAnimationsModule
];
class MaterialModule {
}
MaterialModule.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.0.3", ngImport: i0, type: MaterialModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule });
MaterialModule.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "12.0.0", version: "13.0.3", ngImport: i0, type: MaterialModule, imports: [CommonModule, MatProgressBarModule,
        MatCheckboxModule,
        MatRadioModule,
        MatButtonModule,
        MatIconModule,
        MatProgressBarModule,
        MatProgressSpinnerModule,
        MatDividerModule,
        MatFormFieldModule,
        MatChipsModule,
        MatInputModule,
        MatTooltipModule
        // BrowserAnimationsModule
    ], exports: [MatProgressBarModule,
        MatCheckboxModule,
        MatRadioModule,
        MatButtonModule,
        MatIconModule,
        MatProgressBarModule,
        MatProgressSpinnerModule,
        MatDividerModule,
        MatFormFieldModule,
        MatChipsModule,
        MatInputModule,
        MatTooltipModule
        // BrowserAnimationsModule
    ] });
MaterialModule.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "13.0.3", ngImport: i0, type: MaterialModule, imports: [[
            CommonModule,
            modules
        ], MatProgressBarModule,
        MatCheckboxModule,
        MatRadioModule,
        MatButtonModule,
        MatIconModule,
        MatProgressBarModule,
        MatProgressSpinnerModule,
        MatDividerModule,
        MatFormFieldModule,
        MatChipsModule,
        MatInputModule,
        MatTooltipModule
        // BrowserAnimationsModule
    ] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.0.3", ngImport: i0, type: MaterialModule, decorators: [{
            type: NgModule,
            args: [{
                    declarations: [],
                    imports: [
                        CommonModule,
                        modules
                    ],
                    exports: [modules]
                }]
        }] });

class SpTreeviewModule {
}
SpTreeviewModule.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.0.3", ngImport: i0, type: SpTreeviewModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule });
SpTreeviewModule.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "12.0.0", version: "13.0.3", ngImport: i0, type: SpTreeviewModule, declarations: [SpTreeviewComponent,
        SpTreeviewDropdownComponent,
        SpTreeviewNodeComponent,
        SpTreeviewOverlayComponent], imports: [CommonModule,
        // BrowserAnimationsModule,
        // MatProgressBarModule,
        // MatCheckboxModule,
        // MatRadioModule,
        // MatButtonModule,
        // MatIconModule,
        // MatInputModule,
        // MatFormFieldModule,
        // MatChipsModule,
        // MatDividerModule,
        FormsModule,
        MaterialModule], exports: [SpTreeviewComponent,
        SpTreeviewOverlayComponent,
        SpTreeviewDropdownComponent] });
SpTreeviewModule.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "13.0.3", ngImport: i0, type: SpTreeviewModule, imports: [[
            CommonModule,
            // BrowserAnimationsModule,
            // MatProgressBarModule,
            // MatCheckboxModule,
            // MatRadioModule,
            // MatButtonModule,
            // MatIconModule,
            // MatInputModule,
            // MatFormFieldModule,
            // MatChipsModule,
            // MatDividerModule,
            FormsModule,
            MaterialModule
        ]] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.0.3", ngImport: i0, type: SpTreeviewModule, decorators: [{
            type: NgModule,
            args: [{
                    declarations: [
                        SpTreeviewComponent,
                        SpTreeviewDropdownComponent,
                        SpTreeviewNodeComponent,
                        SpTreeviewOverlayComponent
                    ],
                    imports: [
                        CommonModule,
                        // BrowserAnimationsModule,
                        // MatProgressBarModule,
                        // MatCheckboxModule,
                        // MatRadioModule,
                        // MatButtonModule,
                        // MatIconModule,
                        // MatInputModule,
                        // MatFormFieldModule,
                        // MatChipsModule,
                        // MatDividerModule,
                        FormsModule,
                        MaterialModule
                    ], exports: [SpTreeviewComponent,
                        SpTreeviewOverlayComponent,
                        SpTreeviewDropdownComponent]
                }]
        }] });

/**
 * Generated bundle index. Do not edit.
 */

export { CHECKED, Config, DropdownLevelConfig, INDETERMINATE, Node, NodeLevelConfig, NodeState, SELECT_CHECKBOX, SELECT_NONE, SELECT_RADIO, SpTreeviewComponent, SpTreeviewDropdownComponent, SpTreeviewModule, SpTreeviewNodeComponent, SpTreeviewNodeTemplateContext, SpTreeviewOverlayComponent, TreeLevelConfig, UNCHECKED };
//# sourceMappingURL=sp-treeview-v4.mjs.map
